<?php
/**
 * SMTP
 * 
 * Connect to a SMTP server so you can send emails.
 * Useful when the SMTP server requires an authentication.
 */

include_once(dirname(__FILE__).'/smtpsocket.class.php');

class smtp{

	private $conn, $isAuth = false, $debug = 0;
	public $host = 'localhost',	// default host is "localhost"
		   $port = 25,			// default port is 25
		   $helo = 'localhost',	// some servers demand the hostname (your pc name) as HELO
		   $auth = false,		// false if smtp auth is not required
		   $user,				// if auth is true, this should be set
		   $pass,				// if auth is true, this should be set
		   $timeout = 5;		// the more timeout time, the more waiting time if server is unreachable		
	
	/**
	 * 
	 */
    function __construct($params = array()) {
		// Loading the user's config     	
    	foreach($params as $k=>$v)
    		$this->$k = $v;
    	
    	// Opening a socket connection
    	$this->connect();
    	
    	// Authenticating against the SMTP server if this->auth = true
    	if($this->auth == true){
    		if($this->debug)
    			echo "Auth with :: $this->user:$this->pass\r\n";
    		$this->ehlo();
    		$this->auth();
    	} else {
    		$this->helo();
    	}
    }
    
    /**
     * connect
     * 
     * Opens a socket connection with the SMTP server
     */
    private function connect(){
    	$this->conn = new smtpsocket($this->host, $this->port, $this->timeout);
    	$welcome_msg = $this->conn->retrieve();
  	
    	if($this->debug)
    		echo $welcome_msg['code'],' ',$welcome_msg['msg'],"\n";
    }
    
    /**
     * mail
     * 
     * Mimics the php's mail function
     * 
     * Should remove the $from parameter in the near future 
     */
    public function mail($from, $to, $subject, $message, $headers = array()){
    	// Check authentication
    	if($this->isAuth === false && $this->auth == true)
    		if(!$this->auth())
    			return false;
    	
    	// We inform the server that we begin to send a mail
    	$this->startmail($from);
    	
    	// We inform the server the recipients
    	if(is_array($to))
    		foreach($to as $rec)
    			$this->rcpt($rec);
    	else
    		$this->rcpt($to);
    		
    	// We inform the server we send data
    	if(!$this->data())
    		return false;
    	
    	// Now, we set the message (headers, body, etc)
    	if(is_array($headers))
    		$headers = str_replace("\r\n.", "\r\n..", rtrim(implode("\r\n", $headers)));
    	$body = str_replace("\r\n.", "\r\n..", rtrim($message));
    	$body = ($body[0] == '.')? '.'.$body : $body;
    	
    	// Finally, we send the mail
    	$this->conn->send($headers);	// sends the headers to the server
    	$this->conn->send('');			// tells the server we now send the body
    	$this->conn->send($body);		// sends the body to the server
    	$this->conn->send('.');			// finished
    	
    	$response = $this->conn->retrieve();
    	if($this->debug){
    		echo $response['code'],' :: ',$response['msg'],"\r\n";
    		echo $headers,'<br /><br />',$body;
    	}
    	return ( $response['code'] === '250' );
    }
    
    /**
     * helo
     * 
     * Sends an HELO command against the SMTP server
     */
    private function helo(){
    	$resp = $this->conn->sendAndRetrieve('HELO '.$this->helo);

		if($this->debug)
			echo 'HELO :: ',$resp['code'],' :: ',$resp['msg'],"\r\n";

    	if($resp['code'] === '250')
    		return true;
    	else{
    		throw new exception ('FAILED HELO :: '. $resp['msg']);
    		return false;
    	}
    }
    
    /**
     * ehlo
     * 
     * Sends an EHLO command against the SMTP server
     */
    private function ehlo(){
    	$resp = $this->conn->sendAndRetrieve('EHLO '.$this->helo);
    	
    	if($this->debug)
			echo 'EHLO :: ',$resp['code'],' :: ',$resp['msg'],"\r\n";
    	
    	if($resp['code'] === '250')
    		return true;
    	else{
    		throw new exception ('FAILED EHLO :: '. $resp['msg']);
    		return false;
    	}
    }
    
    /**
     * auth
     * 
     * Sends an AUTH command against the SMTP server
     */
    private function auth(){
    	// If already authenticated or the server doesn't
    	// require authentication, we return true
    	if($this->isAuth === true || $this->auth == false)
    		return true;

		// Else, we try to authenticate and return wether
		// it worked or not
		/* AUTHENTICATION */
		$resp1 = $this->conn->sendAndRetrieve('AUTH LOGIN');
		if($resp1['code'] === '334'){
			/* USERNAME */
			$resp2 = $this->conn->sendAndRetrieve(base64_encode($this->user));
			if($resp2['code'] == '334'){
				/* PASSWORD */
				$resp3 = $this->conn->sendAndRetrieve(base64_encode($this->user));
				if($resp3['code'] === '235' ){
					
					if($this->debug)
						echo 'AUTH1 :: ',$resp1['code'],' :: ',$resp1['msg'],"\r\n",
							 'AUTH2 :: ',$resp2['code'],' :: ',$resp2['msg'],"\r\n",
							 'AUTH3 :: ',$resp3['code'],' :: ',$resp3['msg'],"\r\n";
						
					$this->isAuth = true;
					return true;
				} else {
					throw new exception('FAILED AUTH (Pass) :: '. $resp3['msg'] . ' Params :: user : '.$this->user.' pass : *****');
				}
				/* /PASSWORD */
   			} else {
				throw new exception('FAILED AUTH (User) :: '. $resp2['msg'] . ' Params :: user : '.$this->user.' pass : *****');
			}
			/* /USERNAME */
		} else {
			throw new exception('FAILED AUTH (Auth) :: '. $resp1['msg'] . ' Params :: user : '.$this->user.' pass : *****');
		}
		/* /AUTHENTICATION */
		
		return false;
    }
    
    /**
     * startmail
     * 
     * Sends a MAIL FROM command against the SMTP server
     */
    private function startmail($from){
    	$resp = $this->conn->sendAndRetrieve('MAIL FROM:<'.$from.'>');
    	
    	if($this->debug)
			echo 'MAIL FROM :: ',$resp['code'],' :: ',$resp['msg'],"\r\n";
    	
    	if($resp['code'] === '250' )
			return true;
		else
			return false;
		
    }
    
    /**
     * data
     * 
     * Sends a DATA command against the SMTP server
     */
    private function data(){
    	$resp = $this->conn->sendAndRetrieve('DATA');

		if($this->debug)
			echo 'DATA :: ',$resp['code'],' :: ',$resp['msg'],"\r\n";

    	if($resp['code'] === '354' )
			return true;
		else{
			throw new exception('FAILED DATA :: '. $this->conn->retrieve());
			return false;
		}
    }
    
    /**
     * rcpt
     * 
     * Sends a RCPT TO command agains the SMTP server
     */
    private function rcpt($to){
    	$resp = $this->conn->sendAndRetrieve('RCPT TO:<'.$to.'>');
    	
    	if($this->debug)
			echo 'RCPT :: ',$resp['code'],' :: ',$resp['msg'],"\r\n";
    	
    	if($resp['code'] === '250' )
			return true;
		else{
			throw new exception('FAILED RCPT :: '. $this->conn->retrieve() . ' :: Params = '.$to);
			return false;
		}
    }
    
    /**
     * destruct
     * 
     * Closes the connection
     */
    /**
     * rcpt
     * 
     * Sends a RCPT TO command agains the SMTP server
     */
    public function __destruct(){
    	$resp = $this->conn->sendAndRetrieve("QUIT \r\n");
    	if($this->debug)
    		echo 'QUIT :: ', $resp['code'], ' :: ', $resp['msg'], "\r\n";
    }
}
?>